////////////////////////////////////////////////////////////////////
//                                                                //
//              Copyright (c) 1992   Namir Clement Shammas        //
//                                                                //
//  Version:  1.0.0                                  DATE 5/5/92  //
//                                                                //
//  Purpose: implements a class of doubly-linked lists with       //
//           the following operations and features:               //
//                                                                //
//             + insert new node.                                 //
//             + search for a node with a specific occurrence     //
//               of a key.                                        //
//             + delete a node with a specific occurrence of a    //
//               key.                                             //
//             + traverse the nodes of the lists.                 //
//                                                                //
////////////////////////////////////////////////////////////////////

#include "comndata.h"
#include <string.h>

#ifndef _LIST_HPP
#define _LIST_HPP
template<class T>
class DoubleList
{
  protected:
    struct ListNode {
       T listNodeData;
       ListNode *prevNode;
       ListNode *nextNode;
    };

    ListNode *thisPtr,  // search node pointer
             *nodePtr,  // node-visitation pointer
             *tailPtr,  // pointer to the tailPtr of list
             *headPtr;  // pointer to headPtr of list
    unsigned nodeCount;      // number of nodes
    string80 errMsg;  // error message

    DoubleList<T>& copy(DoubleList<T>&);

  public:
    // ************** state manipulation methods **************
    DoubleList();
    DoubleList(DoubleList<T>& dl)
      { headPtr = NULL; copy(dl); }
    // add virtual ~DoubleList()
    virtual ~DoubleList()
      { clear(); }

    // ****************** state query methods *****************
    bool isListEmpty() const
      { return (nodeCount == 0) ? true : false; }
    unsigned getNodeCount() const
      { return nodeCount; }
    char* getErrMsg();

    // ************** object manipulation methods *************
    virtual bool insert(T&);
    virtual bool insert(T&, T&) ;
    virtual bool remove(T& x, unsigned occur = 1);
    virtual bool search(T& x, unsigned occur = 1);
    virtual bool getFirstNode(T& x);
    bool getPrevNode(T& x);
    bool getPrevNode(T x, T& y);
    virtual bool getNextNode(T& x);
    virtual bool getNextNode(T x, T& y);
    bool getLastNode(T& x);
    bool getData(T& x, 
                    unsigned index = 0,
                    bool updateNodePtr = false);
    virtual void clear();
    DoubleList<T>& operator =(DoubleList<T>& dl)
      { copy(dl); return *this; }
};

template<class T>
class SortedDoubleList  : public DoubleList<T>
{
  public:
    // ************** state manipulation methods **************
    SortedDoubleList () : DoubleList<T>() {}

    SortedDoubleList (SortedDoubleList <T>& dl)
      { headPtr = NULL; copy(dl); }

    virtual ~SortedDoubleList() {}     // Added 1/29/96 by Jeff Nielsen
				       // to prevent compiler bug

    // ************** object manipulation methods *************
    virtual bool insert(T&);
    virtual bool search(T& x, unsigned occur = 1);

    SortedDoubleList <T>& operator =(SortedDoubleList <T>& dl)
      { copy(dl); return *this; }
};

#endif
