/************************************************************
 * sail.h ---- the general header file for SAIL.	    *
 * Jun Yang, November 17, 1994				    *
 ************************************************************/

/* These are needed for making Windows Apps. */
#include <windows.h>
#define GS_MAIN CALLBACK WinMain
#define GS_MAIN_ARGS_F HANDLE hInstance, HANDLE hPrevInstance, \
									LPSTR lpszCmdParam, int nCmdShow
#define GS_MAIN_ARGS_A hInstance, hPrevInstance, lpszCmdParam, nCmdShow


/*--------------- Constants ---------------*/
 
/* node types */
#define BOX		1
#define DABOX		2
#define DDBOX		3
#define TABOXL		4
#define TDBOXL		5
#define TABOXC		6
#define TDBOXC		7
#define CIRCLE		8

/* line size */
#define THINLINE	1
#define THICKLINE	3

/* fill type */
#define FRAMED		0
#define FILLED		1

/* BOOLEAN constants */
#define TRUE	1
#define FALSE   0
#define ON	1
#define OFF	0

/* BINTREE CHILD */
#define LEFTCHILD	0
#define RIGHTCHILD	1

/* Graph Type */
#define DIRECTED	TRUE
#define UNDIRECTED	FALSE

/* Layout Components types */
#define MANUAL   	0
#define LISTACROSS	1
#define LISTDOWN	2
#define CIRCLENET	3
#define TREE		4
#define KKNET		5
#define HIERACHY	6
#define BINTREE		7
#define ARRAYACROSS	8
#define ARRAYDOWN	9

/* graphical attributes */
#define GDIRECTED	0
#define GLAYOUTTYPE	1
#define GNEWLAYOUT	2
#define GAUTORELAYOUT	3
#define GLAYOUTMODE	4
#define GNODETYPE 	11
#define GNODECOLOR	12
#define GNODEFILLED	13
#define GNODEWIDTH	14
#define GNODEHEIGHT	15
#define GNODELINETH	17
#define GNODEPOS	18
#define GNODELABEL	19
#define GEDGELENGTH	21
#define GEDGELINETH	22
#define GEDGEARROW	23
#define GEDGECOLOR	24
#define GEDGELABEL	25

/* layout method */
#define MANULAYOUT      0
#define AUTOLAYOUT      1

#define GAUTOREDRAW	50

/* switches */
#define EDGELABEL	2

/* constant ID  */
#define NULLGRAPHID	0
#define NULLLCID	0

/* process status */
#define RUN   		1 
#define STEP    	2
#define INSNODE		4	
#define DELNODE		8
#define INSEDGE		16
#define DELEDGE		32

/* types of lines */
#define SOLIDLINE	1
#define DASHEDLINE	2
#define DOUBLEDASHLINE	3

/* types of arrows */
#define NOARROW         1
#define ENDARROW        2
#define STARTARROW      3
#define BOTHARROW       4


/* color tables */
#define BLACK           0
#define DGRAY           1
#define MGRAY           2
#define LGRAY           3
#define BLUEGRAY        4
#define LBLUE           5
#define PEACH           6
#define LCYAN           7
#define MCYAN           8
#define GRAY            9
#define MYELLOW         10
#define LYELLOW         11
#define MAGENTA         12
#define DGREEN          13
#define PASGREEN        14

/* types of fonts */

#define SMALLFONT	0

/* menu list */
#define EDITMENU	0
extern char* SW_MENU[];

/* EDITMENU item list */
#define ITEMINSNODE	0	
#define ITEMDELNODE	1
#define ITEMINSEDGE	2
#define ITEMDELEDGE	3
extern char* SW_MENUITEM[];

/*--------------- Types ---------------*/

typedef long	GRAPHID ;
typedef long	LC_ID ;//need to replace LC_ID w/ LC_ID for compat w/win95 jrf
typedef int	LAYOUTCOMPTYPE ;
typedef int*    NODEID ;
typedef int	NODETYPE ;
typedef char* 	LABEL ;
typedef int     FLAG ;
typedef int	INDEX ;
typedef int	IVALUE ;

/*---------------- Function Prototypes ----------------*/

/* Swan initialization and clean up */
bool sw_init(GS_MAIN_ARGS_F) ;
void sw_quit(void) ;
void sw_clear(void) ;
int sw_wait(int) ;

void sw_enablebuttons(int) ;
void sw_disablebuttons(int) ;
bool sw_enablemenuitem(int, int) ;
bool sw_disablemenuitem(int, int) ;

/* Swan graph construction and destruction */
GRAPHID	sw_newgraph(GRAPHID, NODETYPE, FLAG) ;
bool sw_deletegraph(GRAPHID) ;

/* Swan layout component construction and destruction */
LC_ID sw_newlc(GRAPHID, LC_ID, LAYOUTCOMPTYPE) ;
bool sw_deletelc(GRAPHID, LC_ID) ;
bool sw_setcurlc(GRAPHID, LC_ID) ;

/* Swan graph display */
bool sw_displaygraph(GRAPHID) ;
void sw_displayallgraphs(void) ;

/* Swan nodes/edges operation */
bool sw_insertnode(GRAPHID, NODEID, LABEL) ;
bool sw_deletenode(GRAPHID, NODEID) ;
bool sw_insertedge(GRAPHID, NODEID, NODEID, LABEL) ;
bool sw_insertbinedge(GRAPHID, NODEID, NODEID, LABEL, FLAG) ;
bool sw_deleteedge(GRAPHID, NODEID, NODEID);
bool sw_insertnodeedge(GRAPHID, NODEID, LABEL, NODEID, LABEL,
			  LABEL) ;

/* Swan graphic attributes settings */
bool sw_setcurrentattr(INDEX, IVALUE) ;
bool sw_setgraphattr(GRAPHID, INDEX, IVALUE) ;
bool sw_setgraphattr(GRAPHID, INDEX, IVALUE) ;
bool sw_setgraphpos(GRAPHID, int, int) ;
bool sw_getgraphpos(GRAPHID, int *, int *) ;	// Added by Jeff Nielsen
bool sw_setlcattr(GRAPHID, LC_ID, INDEX, IVALUE) ;
bool sw_setnodeattr(GRAPHID, NODEID, INDEX, ...) ;
bool sw_getnodeattr(GRAPHID, NODEID, INDEX, ...) ;
bool sw_setedgeattr(GRAPHID, NODEID, NODEID, INDEX, ...) ;
bool sw_getedgeattr(GRAPHID, NODEID, NODEID, INDEX, ...) ;

/* Swan process controls */
void sw_run(void) ;
void sw_step(void) ;
void sw_break(void) ;

/* Error processing */
int sw_errno(void) ;
bool sw_errmsg(int, char*) ;

/* Input/Output */
void sw_print(LABEL) ;
void sw_getstr(char*) ;

/* pick functions */
bool sw_pickgraph(GRAPHID*) ;
bool sw_picknode(NODEID*) ;
bool sw_pickedge(NODEID*, NODEID*) ;
bool sw_pickpos(int*, int*) ;

bool sw_getpickedgraph(GRAPHID*) ;
bool sw_getpickednode(NODEID*) ;
bool sw_getpickededge(NODEID*, NODEID*) ;
bool sw_getpickedpos(int*, int*) ;

extern char* inttostr(int);
extern int cmp_int(const void*, const void*) ;

/*-------------- Miscellaneous ---------------*/

extern int sc_x_origin ;
extern int sc_y_origin ;
extern float zoom_factor ;

/************************************************************
 * End of sail.h				            *
 ************************************************************/
